@@ -6,7 +6,7 @@ setuptools entry points.
 """
 
 # import udiskie.depend first - for side effects!
-from .depend import has_Notify, has_Gtk, _in_X
+from .depend import has_Notify, has_Gtk, _in_X, _in_Wayland, has_AppIndicator3
 
 import inspect
 import logging.config
@@ -303,8 +303,18 @@ class Daemon(_EntryPoint):
         -p COMMAND, --password-prompt COMMAND   Command for password retrieval
         -P, --no-password-prompt                Disable unlocking
 
-        --notify-command COMMAND                Command to execute on events
-        --no-notify-command                     Disable command notifications
+        --event-hook COMMAND                    Command to execute on events
+        --no-event-hook                         Disable command notifications
+
+        --menu-checkbox-workaround              Use checkbox workaround
+        --no-menu-checkbox-workaround           Disable checkbox workaround
+
+        --menu-update-workaround                Use wayland menu workaround
+        --no-menu-update-workaround             Disable wayland menu workaround
+
+    [DEPRECATED]:
+        --notify-command COMMAND                Renamed to --event-hook
+        --no-notify-command                     Renamed to --no-event-hook
     """
 
     option_defaults = extend(_EntryPoint.option_defaults, {
@@ -312,12 +322,15 @@ class Daemon(_EntryPoint):
         'notify': True,
         'tray': False,
         'menu': 'flat',
-        'appindicator': False,
+        'appindicator': None,
         'file_manager': 'xdg-open',
         'terminal': '',
         'password_prompt': 'builtin:gui',
         'password_cache': False,
         'notify_command': None,
+        'event_hook': None,
+        'menu_checkbox_workaround': None,
+        'menu_update_workaround': None,
     })
 
     option_rules = extend(_EntryPoint.option_rules, {
@@ -333,10 +346,12 @@ class Daemon(_EntryPoint):
         'password_prompt': OptionalValue('--password-prompt'),
         'password_cache': OptionalValue('--password-cache'),
         'notify_command': OptionalValue('--notify-command'),
+        'event_hook': OptionalValue('--event-hook'),
+        'menu_checkbox_workaround': OptionalValue('--menu-checkbox-workaround'),
+        'menu_update_workaround': OptionalValue('--menu-update-workaround'),
     })
 
     def _init(self):
-
         import udiskie.prompt
 
         config = self.config
@@ -375,14 +390,18 @@ class Daemon(_EntryPoint):
             logging.getLogger(__name__).error(libnotify_not_available)
             options['notify'] = False
 
-        if options['tray'] and not _in_X:
-            no_X_session = _(
-                "Not run within X session. "
-                "\nStarting udiskie without tray icon.\n")
-            logging.getLogger(__name__).error(no_X_session)
-            options['tray'] = False
+        show_tray = options['tray'] or options['appindicator']
+        if show_tray and _in_Wayland and options['appindicator'] is None:
+            options['appindicator'] = True
 
-        if options['tray'] and not has_Gtk(3):
+        if show_tray and not (_in_X or _in_Wayland):
+            no_tray_support = _(
+                "Not run within X or Wayland session."
+                "\nStarting udiskie without tray icon.\n")
+            logging.getLogger(__name__).error(no_tray_support)
+            show_tray = False
+
+        if show_tray and not has_Gtk(3):
             gtk3_not_available = _(
                 "Typelib for 'Gtk 3.0' is not available. Possible causes include:"
                 "\n\t- GTK3 is not installed"
@@ -390,7 +409,18 @@ class Daemon(_EntryPoint):
                 "\n\t- GTK3 was built with introspection disabled"
                 "\nStarting udiskie without tray icon.\n")
             logging.getLogger(__name__).error(gtk3_not_available)
-            options['tray'] = False
+            show_tray = False
+
+        if show_tray and options['appindicator'] and not has_AppIndicator3():
+            appindicator_not_available = _(
+                "Typelib for 'AppIndicator3 0.1' is not available. Possible "
+                "causes include:"
+                "\n\t- libappindicator is not installed"
+                "\n\t- the typelib is provided by a separate package"
+                "\n\t- it was built with introspection disabled"
+                "\nStarting udiskie without appindicator icon.\n")
+            logging.getLogger(__name__).error(appindicator_not_available)
+            options['appindicator'] = False
 
         # start components
         tasks = []
@@ -400,12 +430,23 @@ class Daemon(_EntryPoint):
         self.automounter = self._load_automounter(options['automount'])
         self.automounter.activate()
 
+        if options['notify_command']:
+            logging.getLogger(__name__).warn(_(
+                "The 'notify_command' option was renamed to 'event_hook'. "
+                "The old name still works, but may be removed in a future version. "
+                "Please change your command line and config to use the new name."))
+            if options['event_hook'] is None:
+                options['event_hook'] = options['notify_command']
+            else:
+                logging.getLogger(__name__).warn(_(
+                    "Ignoring 'notify_command' in favor of 'event_hook'."))
+
         if options['notify']:
             self.notify.activate()
-        if options['notify_command']:
+        if options['event_hook']:
             # is currently enabled/disabled statically only once:
-            self.notify_command()
-        if options['tray']:
+            self.connect_event_hook(options['event_hook'])
+        if show_tray:
             self.statusicon.activate()
             tasks.append(self.statusicon.instance._icon.task)
         else:
@@ -430,25 +471,29 @@ class Daemon(_EntryPoint):
             timeout=self.config.notifications,
             aconfig=aconfig)
 
-    def notify_command(self):
+    def connect_event_hook(self, command):
         import udiskie.prompt
-        return udiskie.prompt.notify_command(
-            self.options['notify_command'], self.mounter)
+        return udiskie.prompt.connect_event_hook(command, self.mounter)
 
     def _load_statusicon(self):
         import udiskie.tray
         options = self.options
         config = self.config
 
-        if options['tray'] == 'auto':
-            smart = True
-        elif options['tray'] is True:
-            smart = False
-        else:
+        smart = options['tray'] == 'auto'
+        if options['tray'] not in ('auto', True, False):
             raise ValueError("Invalid tray: %s" % (options['tray'],))
         icons = udiskie.tray.Icons(self.config.icon_names)
         actions = udiskie.mount.DeviceActions(self.mounter)
 
+        checkbox_workaround = options['menu_checkbox_workaround']
+        if checkbox_workaround is None:
+            checkbox_workaround = options['appindicator'] and _in_Wayland
+
+        update_workaround = options['menu_update_workaround']
+        if update_workaround is None:
+            update_workaround = options['appindicator'] and _in_Wayland
+
         if options['menu'] == 'flat':
             flat = True
         # dropped legacy 'nested' mode:
@@ -457,8 +502,11 @@ class Daemon(_EntryPoint):
         else:
             raise ValueError("Invalid menu: %s" % (options['menu'],))
 
-        menu_maker = udiskie.tray.UdiskieMenu(self, icons, actions, flat,
-                                              config.quickmenu_actions)
+        menu_maker = udiskie.tray.TrayMenu(
+            self, icons, actions, flat,
+            config.quickmenu_actions,
+            checkbox_workaround=checkbox_workaround,
+            update_workaround=update_workaround)
         if options['appindicator']:
             import udiskie.appindicator
             TrayIcon = udiskie.appindicator.AppIndicatorIcon
